@@ -13,13 +13,18 @@ minimal adaptations so it can live inside the ``sheshe`` package.
 
 from __future__ import annotations
 
-from typing import Callable, Dict, List, Optional, Tuple, Union
+from typing import Any, Callable, Dict, List, Optional, Tuple, Union
 from matplotlib.lines import Line2D
 
 import time
 import numpy as np
 import pandas as pd
 import matplotlib.pyplot as plt
+from pathlib import Path
+import joblib
+
+from .region_interpretability import RegionInterpreter
+from .sheshe import ClusterRegion
 
 from sklearn.ensemble import RandomForestRegressor
 from sklearn.neighbors import NearestNeighbors
@@ -572,6 +577,9 @@ class ShuShu:
         self.classes_: Optional[np.ndarray] = None
         self.model_ = None
         self.mode_: Optional[str] = None
+        self.score_fn_: Optional[Callable[[np.ndarray], np.ndarray]] = None
+        # expose discovered regions from underlying clusterers
+        self.regions_: List[Dict] = []
 
     def _build_score_fns(
         self,
@@ -636,6 +644,8 @@ class ShuShu:
             self.clusters_ = clusterer.clusters_
             self.timings_ = clusterer.timings_
             self.mode_ = "scalar"
+            self.score_fn_ = score_fn
+            self.regions_ = list(clusterer.clusters_)
             return self
 
         self.feature_names_ = (
@@ -673,8 +683,34 @@ class ShuShu:
         self.per_class_ = results
         self.classes_ = classes
         self.mode_ = "multiclass"
+        # flatten regions across classes for convenient access
+        all_regions: List[Dict] = []
+        for info in results.values():
+            clus = info["clusterer"]
+            cls_label = info["label"]
+            for reg in getattr(clus, "clusters_", []) or []:
+                r = dict(reg)
+                r["cluster_id"] = len(all_regions)
+                r["label"] = cls_label
+                all_regions.append(r)
+        self.regions_ = all_regions
         return self
 
+    def fit_predict(self, X: np.ndarray, y: Optional[np.ndarray] = None, **fit_kwargs) -> np.ndarray:
+        """Convenience method that fits and then predicts ``X``.
+
+        Parameters
+        ----------
+        X : array-like
+            Training data.
+        y : array-like, optional
+            Labels for multiclass mode.
+        **fit_kwargs : dict
+            Additional keyword arguments passed to :meth:`fit`.
+        """
+        self.fit(X, y, **fit_kwargs)
+        return self.predict(X)
+
     def predict_proba(self, X: np.ndarray) -> np.ndarray:
         """Return class probabilities for ``X``.
 
@@ -708,6 +744,36 @@ class ShuShu:
                 raise RuntimeError("Clusterer missing for predict")
             return self.clusterer_._assign_labels(X)
 
+    def decision_function(self, X: np.ndarray) -> np.ndarray:
+        """Return raw decision scores for ``X``.
+
+        In multiclass mode this delegates to the underlying ``score_model`` if
+        it provides :meth:`decision_function`. In scalar mode it returns the
+        negative distance to each centroid in the scaled feature space.
+        """
+        if self.mode_ is None:
+            raise RuntimeError("Model not fitted")
+        X = np.asarray(X, dtype=float)
+        if self.mode_ == "multiclass":
+            if self.model_ is None or not hasattr(self.model_, "decision_function"):
+                raise RuntimeError("Underlying model lacks decision_function")
+            return np.asarray(self.model_.decision_function(X))
+        else:
+            if self.clusterer_ is None or self.clusterer_.centroids_ is None:
+                raise RuntimeError("Clusterer missing for decision_function")
+            c = self.clusterer_
+            if c.centroids_.shape[0] == 0:
+                return np.zeros((X.shape[0], 0))
+            Uc = c._to_unit(c.centroids_)
+            if c.selected_dims_.size > 0:
+                Uc = Uc[:, c.selected_dims_]
+            XU = c._to_unit(X)
+            if c.ignored_dims_.size > 0:
+                XU[:, c.ignored_dims_] = c._medians_unit[c.ignored_dims_]
+            ZX = XU[:, c.selected_dims_] if c.selected_dims_.size > 0 else XU
+            dists = np.linalg.norm(ZX[:, None, :] - Uc[None, :, :], axis=2)
+            return -dists
+
     def predict_regions(self, X: np.ndarray):
         """Predict class labels and cluster ids for ``X``.
 
@@ -766,7 +832,17 @@ class ShuShu:
         """
         X = np.asarray(X, dtype=float)
         uniq = np.unique(y)
-        cmap_pts = plt.get_cmap("tab10")
+        palette = [
+            "#e41a1c",
+            "#377eb8",
+            "#4daf4a",
+            "#984ea3",
+            "#ff7f00",
+            "#a65628",
+            "#f781bf",
+            "#999999",
+        ]
+        class_colors = {c: palette[i % len(palette)] for i, c in enumerate(uniq)}
 
         if contour_levels is None:
             contour_levels = np.linspace(0.0, 1.0, 12)
@@ -775,6 +851,7 @@ class ShuShu:
 
         for ci, info in self.per_class_.items():
             label = info["label"]
+            col = class_colors[label]
             c = info["clusterer"]
             dims2 = info["dims2"]
             d1, d2 = int(dims2[0]), int(dims2[1])
@@ -802,11 +879,27 @@ class ShuShu:
 
             zmin, zmax = float(np.nanmin(ZZ)), float(np.nanmax(ZZ))
             frontier_val = 0.5 if (0.5 >= zmin and 0.5 <= zmax) else 0.5 * (zmin + zmax)
-            ax.contour(XX1, XX2, ZZ, levels=[frontier_val], colors="k", linewidths=2)
+            ax.contour(
+                XX1,
+                XX2,
+                ZZ,
+                levels=[frontier_val],
+                colors=[col],
+                linewidths=2,
+            )
 
             for j, yv in enumerate(uniq):
                 mask = y == yv
-                ax.scatter(X[mask, d1], X[mask, d2], s=16, alpha=0.8, label=str(yv), color=cmap_pts(j))
+                ax.scatter(
+                    X[mask, d1],
+                    X[mask, d2],
+                    s=18,
+                    alpha=0.8,
+                    label=str(yv),
+                    color=class_colors[yv],
+                    edgecolor="k",
+                    linewidths=0.3,
+                )
 
             if c.centroids_ is not None and c.centroids_.shape[0] > 0:
                 C2 = c.centroids_[:, [d1, d2]]
@@ -814,8 +907,8 @@ class ShuShu:
                     C2[:, 0],
                     C2[:, 1],
                     marker="X",
-                    s=160,
-                    color="lime",
+                    s=80,
+                    color=col,
                     edgecolor="k",
                     label=f"centro {label} ({c.centroids_.shape[0]})",
                 )
@@ -840,14 +933,16 @@ class ShuShu:
                             np.r_[H[:, 0], H[0, 0]],
                             np.r_[H[:, 1], H[0, 1]],
                             linewidth=2,
-                            color="white",
+                            color=col,
                             alpha=0.9,
                         )
                     except Exception:
                         pass
 
             handles, labels_ = ax.get_legend_handles_labels()
-            frontier_handle = Line2D([], [], color="k", lw=2, label=f"frontera {label} ({frontier_val:.2f})")
+            frontier_handle = Line2D(
+                [], [], color=col, lw=2, label=f"frontera {label} ({frontier_val:.2f})"
+            )
             handles.append(frontier_handle)
             ax.legend(handles=handles)
 
@@ -856,6 +951,175 @@ class ShuShu:
             ax.set_title(f"Prob. clase '{label}' vs ({name1},{name2})")
             fig.tight_layout()
 
+    def plot_pair_3d(
+        self,
+        X: np.ndarray,
+        pair: Tuple[int, int],
+        class_label: Optional[Any] = None,
+        grid_res: int = 50,
+        alpha_surface: float = 0.6,
+        engine: str = "matplotlib",
+    ):
+        """Visualize score surface for a pair of features in 3D."""
+        if self.mode_ is None:
+            raise RuntimeError("Model not fitted")
+        X = np.asarray(X, dtype=float)
+        d1, d2 = pair
+        if self.mode_ == "multiclass":
+            if class_label is None:
+                raise ValueError("class_label required in multiclass mode")
+            info = None
+            for v in self.per_class_.values():
+                if v["label"] == class_label:
+                    info = v
+                    break
+            if info is None:
+                raise ValueError(f"class_label {class_label} not found")
+            c = info["clusterer"]
+            score_fn = info["score_fn"]
+            name1 = self.feature_names_[d1] if self.feature_names_ else f"x{d1}"
+            name2 = self.feature_names_[d2] if self.feature_names_ else f"x{d2}"
+            zlabel = f"P({class_label})"
+            title = f"Prob. clase '{class_label}' vs ({name1},{name2})"
+        else:
+            if self.clusterer_ is None or self.score_fn_ is None:
+                raise RuntimeError("score_fn missing for scalar mode")
+            c = self.clusterer_
+            score_fn = self.score_fn_
+            name1 = self.feature_names_[d1] if self.feature_names_ else f"x{d1}"
+            name2 = self.feature_names_[d2] if self.feature_names_ else f"x{d2}"
+            zlabel = "score"
+            title = f"score vs ({name1},{name2})"
+
+        x1_min, x1_max = X[:, d1].min(), X[:, d1].max()
+        x2_min, x2_max = X[:, d2].min(), X[:, d2].max()
+        pad1 = 0.05 * (x1_max - x1_min + 1e-12)
+        pad2 = 0.05 * (x2_max - x2_min + 1e-12)
+        x1 = np.linspace(x1_min - pad1, x1_max + pad1, grid_res)
+        x2 = np.linspace(x2_min - pad2, x2_max + pad2, grid_res)
+        XX1, XX2 = np.meshgrid(x1, x2)
+
+        base = np.tile(c.medians_, (grid_res * grid_res, 1))
+        base[:, d1] = XX1.ravel()
+        base[:, d2] = XX2.ravel()
+        ZZ = score_fn(base).reshape(grid_res, grid_res)
+
+        if engine == "plotly":
+            try:
+                import plotly.graph_objects as go
+            except Exception as exc:  # pragma: no cover - optional dependency
+                raise ImportError("plotly is required when engine='plotly'") from exc
+            fig = go.Figure(
+                data=[
+                    go.Surface(x=XX1, y=XX2, z=ZZ, colorscale="Viridis", opacity=alpha_surface)
+                ]
+            )
+            fig.update_layout(
+                title=title,
+                scene=dict(xaxis_title=name1, yaxis_title=name2, zaxis_title=zlabel),
+            )
+            return fig
+
+        if engine != "matplotlib":
+            raise ValueError("engine must be 'matplotlib' or 'plotly'")
+
+        fig = plt.figure(figsize=(6, 5))
+        ax = fig.add_subplot(111, projection="3d")
+        ax.plot_surface(XX1, XX2, ZZ, cmap="viridis", alpha=alpha_surface)
+        ax.set_xlabel(name1)
+        ax.set_ylabel(name2)
+        ax.set_zlabel(zlabel)
+        ax.set_title(title)
+        fig.tight_layout()
+        return fig
+
+    def interpretability_summary(self, feature_names: Optional[List[str]] = None) -> pd.DataFrame:
+        """Summarize cluster regions using :class:`RegionInterpreter`."""
+        if self.mode_ is None:
+            raise RuntimeError("Model not fitted")
+
+        if feature_names is None:
+            if self.feature_names_ is not None:
+                feature_names = self.feature_names_
+            else:
+                if self.mode_ == "multiclass":
+                    any_cl = next(iter(self.per_class_.values()))["clusterer"]
+                    d = any_cl.centroids_.shape[1] if any_cl.centroids_ is not None else 0
+                else:
+                    d = self.clusterer_.centroids_.shape[1] if self.clusterer_ and self.clusterer_.centroids_ is not None else 0
+                feature_names = [f"x{j}" for j in range(d)]
+
+        regions: List[ClusterRegion] = []
+        if self.mode_ == "multiclass":
+            for info in self.per_class_.values():
+                cls_label = info["label"]
+                c = info["clusterer"]
+                for cl in c.clusters_ or []:
+                    centroid = cl["centroid"]
+                    ap = cl.get("area_points")
+                    if ap is not None and ap.shape[0] > 0:
+                        vecs = ap - centroid
+                        radii = np.linalg.norm(vecs, axis=1)
+                        mask = radii > 0
+                        dirs = vecs[mask] / radii[mask][:, None]
+                        radii = radii[mask]
+                    else:
+                        dirs = np.zeros((0, centroid.shape[0]))
+                        radii = np.zeros((0,))
+                    inf_pts = centroid[None, :] + dirs * radii[:, None]
+                    regions.append(
+                        ClusterRegion(
+                            cluster_id=int(cl["label"]),
+                            label=cls_label,
+                            center=centroid,
+                            directions=dirs,
+                            radii=radii,
+                            inflection_points=inf_pts,
+                            inflection_slopes=np.zeros_like(radii),
+                            peak_value_real=float(cl.get("score", np.nan)),
+                            peak_value_norm=float(cl.get("decile", np.nan)) / 10.0,
+                        )
+                    )
+        else:
+            if self.clusterer_ is None:
+                return pd.DataFrame()
+            for cl in self.clusterer_.clusters_ or []:
+                centroid = cl["centroid"]
+                ap = cl.get("area_points")
+                if ap is not None and ap.shape[0] > 0:
+                    vecs = ap - centroid
+                    radii = np.linalg.norm(vecs, axis=1)
+                    mask = radii > 0
+                    dirs = vecs[mask] / radii[mask][:, None]
+                    radii = radii[mask]
+                else:
+                    dirs = np.zeros((0, centroid.shape[0]))
+                    radii = np.zeros((0,))
+                inf_pts = centroid[None, :] + dirs * radii[:, None]
+                regions.append(
+                    ClusterRegion(
+                        cluster_id=int(cl["label"]),
+                        label=0,
+                        center=centroid,
+                        directions=dirs,
+                        radii=radii,
+                        inflection_points=inf_pts,
+                        inflection_slopes=np.zeros_like(radii),
+                        peak_value_real=float(cl.get("score", np.nan)),
+                        peak_value_norm=float(cl.get("decile", np.nan)) / 10.0,
+                    )
+                )
+
+        if not regions:
+            return pd.DataFrame()
+
+        interpreter = RegionInterpreter(feature_names=feature_names)
+        cards = interpreter.summarize(regions)
+        try:
+            return RegionInterpreter.to_dataframe(cards)
+        except Exception:
+            return pd.DataFrame(cards)
+
     def summary_tables(self) -> Tuple[pd.DataFrame, pd.DataFrame]:
         rows_c = []
         rows_ct = []
@@ -897,6 +1161,40 @@ class ShuShu:
         per_centroid_df = pd.DataFrame(rows_ct)
         return per_class_df, per_centroid_df
 
+    def save(self, filepath: Union[str, Path]) -> None:
+        """Serialize this instance using ``joblib.dump``.
+
+        Callables used during fitting (e.g. score functions) are not persisted;
+        only the attributes required for inference are stored."""
+
+        state: Dict[str, Any] = {
+            "random_state": self.random_state,
+            "feature_names_": self.feature_names_,
+            "clusterer_": self.clusterer_,
+            "per_class_": {
+                k: {**{kk: vv for kk, vv in v.items() if kk != "score_fn"}, "score_fn": None}
+                for k, v in self.per_class_.items()
+            },
+            "classes_": self.classes_,
+            "model_": self.model_,
+            "mode_": self.mode_,
+            "score_fn_": None,
+            "centroids_": getattr(self, "centroids_", None),
+            "clusters_": getattr(self, "clusters_", None),
+            "timings_": getattr(self, "timings_", None),
+        }
+        joblib.dump(state, filepath)
+
+    @classmethod
+    def load(cls, filepath: Union[str, Path]) -> "ShuShu":
+        """Load a previously saved instance with :meth:`save`."""
+
+        state = joblib.load(filepath)
+        obj = cls(random_state=state.get("random_state"))
+        for k, v in state.items():
+            setattr(obj, k, v)
+        return obj
+
 # =============================================================================
 #                               EJEMPLOS DE USO
 # =============================================================================
